"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZipReader = void 0;
const yauzl_1 = __importDefault(require("yauzl"));
class ZipReader {
    constructor() {
        this.files = [];
    }
    readZip(file) {
        return new Promise((res, rej) => {
            yauzl_1.default.fromBuffer(file, { lazyEntries: true }, (err, zipfile) => {
                if (err) {
                    console.log('Error leyendo zip');
                    rej(err);
                }
                ;
                zipfile.readEntry();
                zipfile.on("entry", (entry) => {
                    const { fileName } = entry;
                    const isDir = this.isDirectory(fileName);
                    const isZip = this.isZip(fileName);
                    if (isDir)
                        return zipfile.readEntry();
                    zipfile.openReadStream(entry, async (err, readStream) => {
                        let file = await this.streamToBuffer(readStream);
                        file = file.slice(0, -1);
                        if (isZip)
                            await this.readZip(file);
                        if (!isZip)
                            this.files.push({
                                filename: fileName,
                                value: file,
                                extension: this.getExtension(fileName)
                            });
                        zipfile.readEntry();
                    });
                });
                zipfile.on("end", () => {
                    res(this.files);
                });
            });
        });
    }
    streamToBuffer(stream) {
        const bufs = [];
        stream.on('data', function (d) { bufs.push(d); });
        return new Promise(res => {
            stream.on("end", function () {
                res(Buffer.concat(bufs));
            });
        });
    }
    isZip(filename) {
        return Boolean(filename.match(/.zip$/));
    }
    isDirectory(filename) {
        return /\/$/.test(filename);
    }
    getExtension(filename) {
        const res = filename.match(/\.([^\.]*)$/);
        if (res && Number(res?.length) > 1) {
            return res[1];
        }
        return '';
    }
}
exports.ZipReader = ZipReader;
// yauzl.fromBuffer(file, {lazyEntries: true}, (err, zipfile) => {
//     if (err) {
//         console.log('Error leyendo zip inicial');
//         throw err;
//     };
//     zipfile.readEntry();
//     zipfile.on("entry", function(entry) {
//         zipfile.readEntry();
//         const bufs: Buffer[] = [];
//         zipfile.openReadStream(entry, function (err, readStream) {
//             if (err) {
//                 console.log('Error en stream');
//                 throw err;
//             }
//             readStream.on('data', function(d){ bufs.push(d); });
//             readStream.on("end", function() {
//                 let buf = Buffer.concat(bufs);
//                 buf = buf.slice(0, -1)
//                 yauzl.fromBuffer(buf, {}, (err, zipfile) => {
//                     if(err) {
//                         console.log('Error leyendo zip anidado');
//                     };
//                     zipfile.on('entry', function (entry) {
//                         console.log(entry);
//                     })
//                 })
//                 zipfile.readEntry();
//             });
//         })
//     })
// });
//# sourceMappingURL=ZipReader.js.map