"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UploadToCsv = exports.tempFile = exports.outputFolder = void 0;
const stream_1 = require("stream");
const zlib_1 = require("zlib");
const Formatter_1 = require("./Formatter/Formatter");
const Filter_1 = require("./Filter/Filter");
const readline_1 = __importDefault(require("readline"));
const buffer_1 = require("buffer");
const objects_to_csv_1 = __importDefault(require("objects-to-csv"));
const iterator_1 = __importDefault(require("./config/iterator"));
const GlobalConfig_1 = require("../../../GlobalConfig");
const fs_1 = __importDefault(require("fs"));
const fs_2 = require("fs");
const path_1 = __importDefault(require("path"));
exports.outputFolder = path_1.default.join(__dirname, 'tmp');
exports.tempFile = path_1.default.join(exports.outputFolder, 'temp.csv');
class UploadToCsv {
    static clean() {
        if ((0, fs_2.existsSync)(exports.tempFile))
            (0, fs_2.unlinkSync)(exports.tempFile);
    }
    async process(file, data) {
        let csvRows = [];
        const stream = stream_1.Readable.from(file);
        const arrFile = await this.gzipToLines(stream);
        for (const line of arrFile) {
            const formatted = Formatter_1.Formatter.format(line);
            if (!Filter_1.Filter.isValid(formatted))
                continue;
            let id_fk_municipio = Number(String(data.codDele).padStart(3, "0").concat(String(data.codMuni).padStart(3, "0")));
            const object = { id_fk_municipio, id_provincia: data.codDele, type: data.codTipo, ...formatted };
            if (iterator_1.default.maxLines <= csvRows.length) {
                await this.arrayObjectsToCsv(csvRows);
                csvRows.length = 0;
            }
            csvRows.push(object);
        }
        if (csvRows.length > 0)
            await this.arrayObjectsToCsv(csvRows);
        csvRows.length = 0;
        return file;
    }
    async upload(data) {
        const streamCsv = fs_1.default.createReadStream(exports.tempFile);
        for (let serviceUpload of GlobalConfig_1.GlobalConfig.getUploadServices()) {
            const service = new serviceUpload();
            await service.upload(streamCsv, GlobalConfig_1.GlobalConfig.getPathDownload(data.Denominacion + '.csv', service, {
                Denominacion: data.Denominacion,
                Codigo: 0,
                __type: ''
            }));
        }
    }
    async arrayObjectsToCsv(objects) {
        const csv = new objects_to_csv_1.default(objects);
        if (!(0, fs_2.existsSync)(exports.outputFolder))
            (0, fs_2.mkdirSync)(exports.outputFolder, { recursive: true });
        await csv.toDisk(exports.tempFile, { append: true });
    }
    async gzipToLines(stream) {
        const arrLines = [];
        const gunzip = (0, zlib_1.createGunzip)();
        const lineReader = readline_1.default.createInterface({
            input: stream.pipe(gunzip)
        });
        let n = 0;
        return await new Promise((res, rej) => {
            lineReader.on('line', async (line) => {
                line = line.trim();
                line = this.convertEncodingLatinToUtf8(line);
                line = line.replace('ï¿½', 'ñ').replace('ï¿½', 'Ñ');
                if (n != 0)
                    arrLines.push(line);
                n += 1;
            });
            lineReader.on('error', (err) => {
                {
                    rej(err);
                }
            });
            lineReader.on('close', () => {
                res(arrLines);
            });
        });
    }
    convertEncodingLatinToUtf8(string) {
        return (0, buffer_1.transcode)(Buffer.from(string), 'latin1', 'utf8').toString('utf8');
    }
}
exports.UploadToCsv = UploadToCsv;
//# sourceMappingURL=UploadToCsv.js.map