"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getAxios_1 = __importDefault(require("./getAxios"));
const puppeteer_extra_1 = __importDefault(require("puppeteer-extra"));
const puppeteer_extra_plugin_recaptcha_1 = __importDefault(require("puppeteer-extra-plugin-recaptcha"));
const sleep_1 = __importDefault(require("../utils/sleep"));
const puppeteer_1 = __importDefault(require("../config/puppeteer"));
const global_1 = require("../config/global");
const selectors_1 = __importDefault(require("../config/selectors"));
const GlobalConfig_1 = require("../GlobalConfig");
const axios = (0, getAxios_1.default)();
class Puppeteer {
    constructor(browser = null, page = null, arrayCookies = []) {
        this.browser = browser;
        this.page = page;
        this.arrayCookies = arrayCookies;
    }
    async init() {
        this.config();
        this.browser =
            this.browser ?? (await puppeteer_extra_1.default.launch(puppeteer_1.default));
        this.page = this.page ?? (await this.browser?.newPage());
        await this.page.setExtraHTTPHeaders({
            "Accept-Language": "es-AR,es-ES;q=0.9,es-US;q=0.8,es-419;q=0.7,es;q=0.6",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36",
            "sec-ch-ua": '"Not_A Brand";v="99", "Google Chrome";v="109", "Chromium";v="109""'
        });
    }
    config() {
        this.plugins();
    }
    plugins() {
        puppeteer_extra_1.default.use((0, puppeteer_extra_plugin_recaptcha_1.default)({
            provider: {
                id: "2captcha",
                token: GlobalConfig_1.GlobalConfig.getCaptchaToken(),
            },
            visualFeedback: true,
        }));
    }
    async authenticate() {
        await this.init();
        this.page?.setDefaultTimeout(60000);
        await this.page?.goto(global_1.global.authUrl);
        //Go to options to login
        await this.page?.waitForSelector(selectors_1.default.buttonKeyLogin);
        await this.page?.click(selectors_1.default.buttonKeyLogin);
        //Select key login
        await this.page?.waitForNavigation();
        await this.page?.screenshot({ path: "screenshot.png" });
        await this.page?.waitForSelector(selectors_1.default.buttonAccessKeyLogin);
        await this.page?.click(selectors_1.default.buttonAccessKeyLogin);
        //Fill form and submit
        await this.page?.waitForSelector(selectors_1.default.inputCredentials);
        const credentials = GlobalConfig_1.GlobalConfig.getCredentials();
        await this.page?.evaluate(({ selector, credentials, submitButton }) => {
            const [dniInput, passwordInput] = Array.from(document.querySelectorAll(selector));
            dniInput.value = credentials.dni ?? '';
            passwordInput.value = credentials.password ?? '';
            const button = document.querySelector(submitButton);
            button?.click();
        }, { selector: selectors_1.default.inputCredentials, credentials, submitButton: selectors_1.default.submitButton });
        try {
            await this.page?.waitForSelector(selectors_1.default.logoutButton);
        }
        catch { }
        ;
        const client = await this.page?.target().createCDPSession();
        const cookies = (await client?.send("Network.getAllCookies"))?.cookies;
        let stringCookies = "";
        let arrayCookies = [];
        cookies?.forEach((cookie) => arrayCookies.push({
            name: cookie.name,
            value: cookie.value,
        }));
        cookies?.forEach((cookie) => (stringCookies += `${cookie.name}=${cookie.value};`));
        this.browser?.close();
        axios.defaults.headers.common["Cookie"] = stringCookies;
        return { arrayCookies };
    }
    async getToken() {
        await this.init();
        await this.page?.goto(global_1.cat.scrapUrl);
        await this.page?.setCookie(...Object(this.arrayCookies));
        await this.page?.goto(global_1.cat.scrapUrl);
        if (this.page?.hasOwnProperty('solveRecaptchas')) {
            const fixedPageCaptcha = this.page;
            if (typeof fixedPageCaptcha.solveRecaptchas == 'function') {
                await fixedPageCaptcha.solveRecaptchas();
            }
        }
        await this.page?.waitForNavigation();
        await (0, sleep_1.default)(2000);
        const tokenSelector = "#ctl00_Contenido_hdnTokenPeticion";
        await this.page?.waitForSelector(tokenSelector);
        const token = await this.page?.evaluate((selector) => {
            return document.querySelector(selector)?.value;
        }, tokenSelector);
        this.browser?.close();
        return token;
    }
    close() {
        this.browser?.close();
    }
}
exports.default = Puppeteer;
//# sourceMappingURL=Puppeteer.js.map