"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getAxios_1 = __importDefault(require("../getAxios"));
const url_1 = require("url");
const global_1 = require("../../config/global");
const regex_1 = require("../../config/regex");
const jsdom_1 = require("jsdom");
const delegations_1 = require("../../config/delegations");
const scrap_1 = __importDefault(require("../../config/scrap"));
const GlobalConfig_1 = require("../../GlobalConfig");
const Config_1 = require("../../interfaces/Config");
const axios = (0, getAxios_1.default)();
class Municipio {
    constructor(getToken) {
        this.getToken = getToken;
        this.type = GlobalConfig_1.GlobalConfig.getType();
    }
    async getFile(data) {
        let { codMuni, codDele } = data;
        const deleIndex = Object.entries(delegations_1.delegations).find(([i, value]) => codDele == Number(i));
        const delegation = deleIndex != undefined ? deleIndex[1] : null;
        if (delegation &&
            (delegation.municipios.includes(codMuni) ||
                delegation.municipios.length == 0)) {
            data.codDele = delegation.value;
        }
        const idpet = await this.getIdPet(data);
        const res = await axios.post(this.type == "SF" ? global_1.shap.downloadUrl : global_1.cat.downloadUrl, { ...data, idpet }, {
            responseType: "arraybuffer",
        });
        let buff = Buffer.from(res.data, 'utf-8');
        for (let servicePreprocessor of (GlobalConfig_1.GlobalConfig.getPreprocessorServices() ?? [])) {
            try {
                const service = new servicePreprocessor();
                buff = await service.process(buff, data);
            }
            catch (err) {
                await GlobalConfig_1.GlobalConfig.callbackCanceled();
                console.log('Error en preprocesador:', servicePreprocessor.constructor.name, err);
            }
        }
        return buff;
    }
    async getIdPet(data) {
        let resGetFormItems = await this.getFormItems(data);
        if (resGetFormItems.data.match(/(Object moved|Sistema no disponible)/)) {
            data.tokenPeticion = encodeURIComponent(await this.getToken());
            resGetFormItems = await this.getFormItems(data);
        }
        const { document, FormData } = new jsdom_1.JSDOM(resGetFormItems.data).window;
        const form = document.querySelector("form") ?? undefined;
        const formData = new FormData(form);
        formData.append("ctl00$Contenido$btnAceptar", "Descargar producto");
        const queryString = new url_1.URLSearchParams(formData).toString();
        const resPetId = await axios.post(global_1.global.formUrl, queryString);
        const idpet = resPetId.data.match(regex_1.idPetMatcher)[1];
        return idpet;
    }
    async getFormItems(data) {
        return await axios.post(global_1.global.formUrl, {
            hdTipologia: data.codTipo,
            hdDelegacion: data.codDele,
            hdMunicipio: data.codMuni,
            hdFecha: data.fecha,
            hdRutaCHC: "",
            hdCapas: this.type == Config_1.ScrapType.SF ? (GlobalConfig_1.GlobalConfig.getCapas() ?? scrap_1.default.capas).join(",") : "",
            hdInfo: "",
            hdUrlRetorno: `/DescargaDatos/SECDescarga${this.type == Config_1.ScrapType.SF ? "CartoSF" : this.type}.aspx`,
            hdUrlDescarga: `SECZIP${this.type == Config_1.ScrapType.SF ? "CartoSF" : this.type}.aspx`,
            hdOrigen: this.type,
            hdTokenPeticion: decodeURIComponent(data.tokenPeticion),
        });
    }
}
exports.default = Municipio;
//# sourceMappingURL=Municipio.js.map