"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const getAxios_1 = __importDefault(require("../getAxios"));
const global_1 = require("../../config/global");
const Municipio_1 = __importDefault(require("./Municipio"));
const sleep_1 = __importDefault(require("../../utils/sleep"));
const regex_1 = require("../../config/regex");
const Puppeteer_1 = __importDefault(require("../Puppeteer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const scrap_1 = __importDefault(require("../../config/scrap"));
const GlobalConfig_1 = require("../../GlobalConfig");
const Config_1 = require("../../interfaces/Config");
const axios = (0, getAxios_1.default)();
class Provincia {
    constructor(info, zip, arrayCookies) {
        this.municipios = [];
        this.info = info;
        this.zip = zip;
        this.arrayCookies = arrayCookies;
        this.type = GlobalConfig_1.GlobalConfig.getType();
    }
    async json() {
        const municipalities = await this.getMunicipios();
        const pathJson = path_1.default.join(__dirname, "..", "..", "json", "provincias.json");
        const rawdata = fs_1.default.readFileSync(pathJson);
        const jsonData = JSON.parse(rawdata.toString());
        jsonData[this.info.Denominacion] = {
            data: this.info,
            municipios: municipalities,
        };
        fs_1.default.writeFileSync(pathJson, JSON.stringify(jsonData));
    }
    async scrap() {
        const municipalities = await this.getMunicipios();
        await this.chunkScrap(municipalities, Number(process.env.CHUNK_SIZE ?? 15));
        const status = GlobalConfig_1.GlobalConfig.getStatus();
        console.log(this.info.Denominacion + (status == Config_1.Status.CANCELED ? " Cancelada" : " Completada"));
    }
    async chunkScrap(municipalities, chunkSize) {
        let promises = [];
        const token = await this.getToken();
        if (!token || token.length < 5)
            throw Error("Invalid token");
        for (let municipioInfo of municipalities) {
            if (GlobalConfig_1.GlobalConfig.getStatus() == Config_1.Status.CANCELED)
                return;
            const { Codigo: municipioCode, Denominacion: municipioName } = municipioInfo;
            const { Codigo: provinciaCode, Denominacion: provinciaName } = this.info;
            const data = {
                codDele: provinciaCode,
                codMuni: municipioCode,
                nameMuni: municipioName,
                nameProv: provinciaName,
                fecha: 20220722,
                PagLlamante: "SECFormularioDescargas",
                Recarga: "N",
                tokenPeticion: encodeURIComponent(token),
            };
            const municipio = new Municipio_1.default(async () => await this.getToken());
            const urbanPromise = municipio.getFile({
                ...data,
                codTipo: "U" + (this.type == "SF" ? "A" : ""),
            });
            const rusticPromise = municipio.getFile({
                ...data,
                codTipo: "R" + (this.type == "SF" ? "A" : ""),
            });
            let urbanFolderPath = `Urbano`;
            let rusticFolderPath = `Rustico`;
            if (!GlobalConfig_1.GlobalConfig.inSeparatedZips()) {
                urbanFolderPath += `/${provinciaName}`;
                rusticFolderPath += `/${provinciaName}`;
            }
            urbanFolderPath += `/${municipioName}`;
            rusticFolderPath += `/${municipioName}`;
            const promiseFilesData = Promise.all([
                (async () => ({
                    file: await urbanPromise,
                    folder: urbanFolderPath,
                    filename: provinciaName +
                        "-" +
                        municipioName +
                        (this.type == "SF" ? scrap_1.default.extensions.shap : scrap_1.default.extensions.cat),
                }))(),
                (async () => ({
                    file: await rusticPromise,
                    folder: rusticFolderPath,
                    filename: provinciaName +
                        "-" +
                        municipioName +
                        (this.type == "SF" ? scrap_1.default.extensions.shap : scrap_1.default.extensions.cat),
                }))(),
            ]);
            promises.push(promiseFilesData);
            if (promises.length == chunkSize) {
                const timeBetweenChunks = process.env.MS_BETWEEN_CHUNKS ?? 0;
                await this.saveZipsOfManyPromises(promises);
                console.log("Chunk terminado");
                await (0, sleep_1.default)(Number(timeBetweenChunks));
                promises = [];
            }
        }
        await this.saveZipsOfManyPromises(promises);
        promises = [];
    }
    async saveZipsOfManyPromises(promises) {
        let files = [];
        try {
            files = await Promise.allSettled(promises);
        }
        catch (err) {
            console.trace(err);
        }
        if (!files)
            return;
        files.forEach((files) => {
            if (files.status == "rejected")
                return;
            const [urbanFileData, rusticFileData] = files.value;
            this.saveZips({
                urban: {
                    file: urbanFileData.file,
                    folder: urbanFileData.folder,
                    filename: urbanFileData.filename,
                },
                rustic: {
                    file: rusticFileData.file,
                    folder: rusticFileData.folder,
                    filename: rusticFileData.filename,
                },
            });
        });
    }
    saveZips(files) {
        const { urban, rustic } = files;
        this.zip.addFile(urban.folder, urban.file, urban.filename);
        this.zip.addFile(rustic.folder, rustic.file, rustic.filename);
    }
    async getMunicipios() {
        const getMunicipiosUrl = global_1.cat.scrapUrl + "/ObtenerMunicipios";
        const res = await axios.post(getMunicipiosUrl, { filtro: "", provincia: this.info.Codigo }, {
            headers: {
                Accept: "application/json, text/javascript, */*; q=0.01",
                "Content-Type": "application/json; charset=UTF-8",
            },
        });
        return res.data.d;
    }
    async getToken() {
        const res = await axios.get(global_1.cat.scrapUrl);
        const html = res.data;
        let token = html.match(regex_1.htmlTokenMatcher)[1];
        if (token === "U") {
            let puppeteer;
            try {
                puppeteer = new Puppeteer_1.default(null, null, this.arrayCookies);
                token = await puppeteer.getToken();
            }
            catch (err) {
                puppeteer?.close();
                throw Error("Getting token error");
            }
        }
        return token;
    }
}
exports.default = Provincia;
//# sourceMappingURL=Provincia.js.map