"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Scraper = void 0;
const getAxios_1 = __importDefault(require("../getAxios"));
const global_1 = require("../../config/global");
const scrap_1 = require("../../config/scrap");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const Zip_1 = __importDefault(require("../Zip"));
const Provincia_1 = __importDefault(require("./Provincia"));
const Puppeteer_1 = __importDefault(require("../Puppeteer"));
const Config_1 = require("../../interfaces/Config");
const GlobalConfig_1 = require("../../GlobalConfig");
const pathJson = path_1.default.join(__dirname, "..", "..", "json", "provincias.json");
const axios = (0, getAxios_1.default)();
class Scraper {
    constructor(config) {
        this.zipService = new Zip_1.default("scrap.zip");
        GlobalConfig_1.GlobalConfig.set(config);
    }
    async init() {
        const onlyConfig = (GlobalConfig_1.GlobalConfig.getOnly() ?? scrap_1.only);
        const excludesConfig = (GlobalConfig_1.GlobalConfig.getExcludes() ?? scrap_1.excludes);
        const puppeteer = new Puppeteer_1.default();
        try {
            const { arrayCookies } = await puppeteer.authenticate();
            this.arrayCookies = arrayCookies;
        }
        catch (err) {
            console.log("Error autenticando:");
            await GlobalConfig_1.GlobalConfig.callbackCanceled();
            if (err instanceof Error)
                console.trace(err.message);
            puppeteer.close();
            return;
        }
        const intervalAuth = setInterval(() => puppeteer.authenticate, 45 * 60 * 1000);
        const provincias = await this.getProvincias();
        if (GlobalConfig_1.GlobalConfig.getType() === "JSON") {
            fs_1.default.writeFileSync(pathJson, "{}");
        }
        for (let provinciaInfo of provincias) {
            if (GlobalConfig_1.GlobalConfig.getStatus() == Config_1.Status.CANCELED) {
                console.log("Scrap cancelado");
                await GlobalConfig_1.GlobalConfig.callbackCanceled();
                return;
            }
            ;
            if (onlyConfig.length > 0 && !onlyConfig.includes(provinciaInfo.Denominacion))
                continue;
            if (excludesConfig.length > 0 &&
                excludesConfig.includes(provinciaInfo.Denominacion))
                continue;
            const provincia = new Provincia_1.default(provinciaInfo, this.zipService, this.arrayCookies);
            if (GlobalConfig_1.GlobalConfig.getType() === "JSON") {
                await provincia.json();
                continue;
            }
            try {
                console.log("Init scrap", provinciaInfo.Denominacion);
                await provincia.scrap();
                if (GlobalConfig_1.GlobalConfig.inSeparatedZips() && Boolean(GlobalConfig_1.GlobalConfig.zipUpload())) {
                    this.zipService.setFilename(provinciaInfo.Denominacion +
                        "-" +
                        GlobalConfig_1.GlobalConfig.getType() +
                        ".zip");
                    await this.zipService.upload();
                    this.zipService.clear();
                }
                if (GlobalConfig_1.GlobalConfig.preprocessorUpload()) {
                    for (let servicePreprocessor of (GlobalConfig_1.GlobalConfig.getPreprocessorServices() ?? [])) {
                        const service = new servicePreprocessor();
                        try {
                            await service.upload(provinciaInfo);
                        }
                        catch (err) {
                            console.log('Error subiendo preprocesado: ');
                            await GlobalConfig_1.GlobalConfig.callbackCanceled();
                            if (err instanceof Error)
                                console.trace(err.message);
                        }
                    }
                }
                await GlobalConfig_1.GlobalConfig.callbackDownloadedZip(provinciaInfo);
            }
            catch (err) {
                console.log('Error obteniendo scrap: ');
                await GlobalConfig_1.GlobalConfig.callbackCanceled();
                if (err instanceof Error)
                    console.trace(err.message);
                continue;
            }
        }
        if (GlobalConfig_1.GlobalConfig.getType() !== "JSON" &&
            !GlobalConfig_1.GlobalConfig.inSeparatedZips()
            && Boolean(GlobalConfig_1.GlobalConfig.zipUpload())) {
            await this.zipService.upload();
        }
        console.log("Scrapeado correctamente");
        clearInterval(intervalAuth);
    }
    async close() {
        GlobalConfig_1.GlobalConfig.cancel();
    }
    async getProvincias() {
        const getStatesUrl = global_1.cat.scrapUrl + "/ObtenerProvincias";
        const res = await axios.post(getStatesUrl, { filtro: "" }, {
            headers: {
                Accept: "application/json, text/javascript, */*; q=0.01",
                "Content-Type": "application/json; charset=UTF-8",
            },
        });
        return res.data.d;
    }
}
exports.Scraper = Scraper;
//# sourceMappingURL=Scraper.js.map